%------------------------------------------------------------------------%

                       % INFORMACIN RELEVANTE %

%Este programa fue desarrollado por Roberto de Jess Barraza Mendoza con el
%fin de obtener el grado de licenciado en Biologa por la modalidad 
%de tesis bajo el escrito nombrado "Elaboracin y validacin de una  
%herramienta digital para el anlisis de ritmos biolgicos a multiescala en 
%la plataforma de MATLAB" bajo la supervisin de la Dra. Pilar Durn 
%Hernndez. 

%Facultad de Ciencias, UNAM, 2019.

%Para mayor informacin consultar la publicacin de dicha tesis en la base
%de datos TESIUNAM.

%Este trabajo fue desarrollado con fines acadmicos sin nimo de lucro.
%Cualquier interesado en utilizarlo o modificarlo puede llevarlo a cabo
%mientras no infringa ningn principio moral o legal que pudiese afectar a
%los(las) relacionados(as) con el proyecto y se den los crditos necesarios
%en su utilizacin.

%Para su desarrollo fueron utilizados paquetes de herramientas propios de
%la plataforma de MATLAB y algunas otras fuentes que sern referidas en sus
%respectivas secciones, sin embargo, se manifiesta la autora total de este
%trabajo.

%Datos de contacto:
%Roberto Barraza
%r.barrazamendoza@gmail.com

%------------------------------------------------------------------------%

              % Biological Chrono Analysis (BiCA) v1.0 %

%Este archivo (BiCA.m) contiene la ejecucin de la interfaz principal. 

%Ejectese para inicializar el programa.

%------------------------------------------------------------------------%

function varargout = BiCA(varargin)
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @BiCA_OpeningFcn, ...
                   'gui_OutputFcn',  @BiCA_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

function BiCA_OpeningFcn(hObject, eventdata, handles, varargin)

handles.output = hObject;

guidata(hObject, handles);


% UIWAIT makes BiCA wait for user response (see UIRESUME)
% uiwait(handles.figure1);

function varargout = BiCA_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

%-------------------- Nueva seal -------------------------- %
function pushNuevaS_Callback(hObject, eventdata, handles) 
    %Limpiar variables
    clear handles.valores;            
    clear handles.nomb;                
    clear handles.vectt;
    
    %Seleccionar archivo
    [archivo,ruta]=uigetfile('*.txt','Abrir archivo');
    if archivo==0
        return;
    else        
        %Obtener datos del archivo
        fid =fopen([ruta archivo],'r');
        formatSpec = '%s';
        N = 1;
        A = textscan(fid,formatSpec,N, 'delimiter', '\n');

        formatSpec = '%f';
        B = textscan(fid,formatSpec,'headerlines',0,'delimiter','@');

        nomb = A {1}; 
        C = cell2mat (B);

        ind=find(isnan(C));        
        Z = {};
            
        if ind ~= 0 
            for i = 1 : length (ind)-1
                Z {i+1} = C (ind(i)+1:ind(i+1)-1);
            end
            Z {1} = C (1:ind(1)-1);
            Z {end+1} = C (ind(end)+1:end);
        else
            Z = {C(1:end)};
        end
        
        Z = cell2mat(Z);
        sz = size(Z);
        
        %Abrir gui para tomar datos y crear vector de tiempo
        a = guiNuevaS();
        waitfor(a,'Visible','off')
        set(a,'Visible','off')
        hr = getappdata (guiNuevaS,'hr'); dxd = getappdata (guiNuevaS,'dx'); fm = getappdata (guiNuevaS,'fm'); set(a,'Visible','off')
        close (a); clear a;
        
        if hr == 24
            hr = 0;            
        end
        
        %Filtrar los datos
        % Z (Z<0) = 0;
        
        %Algoritmo para establecer el vector de tiempo 
        if fm == 0
            base = 0; 
            t = hr*(1/24);
            for i = 1 : sz(1) -1
                t(end+1) = base + (mod(hr + (24/dxd)*i,24)*1/24);
                if mod (t(end),1) == 0
                    t (end) = base + 1;
                    base = base + 1;
                end
            end
        else 
            t = 0:1/fm:(sz(1)-1)*1/fm;
        end        
        
        tt = t*24; % t, datos para grficas; tt, datos para anlisis
      
        %Experimental tabla    
        scrsz = get(groot,'ScreenSize');
        tab = figure('Name', 'Tabla', ...
            'OuterPosition',[scrsz(3)/2 scrsz(4)/1.9 scrsz(3)/2 scrsz(4)/2.1]);

        %Crear tabla
        uitable('Parent',tab, ...
            'Position', [scrsz(3)*0.012 scrsz(4)*0.04 scrsz(3)*0.465 scrsz(4)/3], ...
            'Data', Z, ...
            'ColumnEditable', true, ...            
            'RowName', []);
        
        %Guardar datos en handles
        handles.valores = Z;                
        handles.vectt = t';
        handles.anat = tt';
        handles.nomb = nomb;
                
        %Guardar datos en aplicacin
        setappdata (BiCA,'valores',handles.valores);
        setappdata (BiCA,'vectt',handles.vectt);
        setappdata (BiCA,'anat',handles.anat);
        setappdata (BiCA,'nomb',handles.nomb);     
        setappdata (BiCA,'dxd',dxd);   
        setappdata (BiCA,'fm',fm);  
        setappdata (BiCA,'datosMin',min(handles.valores));
        setappdata (BiCA,'datosMax',max(handles.valores));
        setappdata (BiCA,'vecttIni',handles.vectt(1));
        setappdata (BiCA,'vecttFin',handles.vectt(end));
        
        guidata(hObject, handles);
        
        if sz(2) > 1
            grupos();
        end
    end
    
    
%---------------------- Graficar --------------------%
function pushbuttonGraficar_Callback(hObject, eventdata, handles)      
    %Obtener tamao de pantalla
    scrsz = get(groot,'ScreenSize');    
    figure('Name', 'Grfica', ...
    'OuterPosition',[scrsz(3)/2 scrsz(1)*30 scrsz(3)/2 scrsz(4)/2]);
    
    plot (handles.vectt,handles.valores,'k');
    xlim ([handles.vectt(1) handles.vectt(end)]);
    
    xlabel('Tiempo')
    title(handles.nomb)
    
% ----------------------- TRF ------------------------------ %
function pushbuttonFFT_Callback(hObject, eventdata, handles)
    %Hacer ventana
    scrsz = get(groot,'ScreenSize');
    graf = figure('Name','TRF', ...
    'OuterPosition',[scrsz(3)/2 scrsz(1)*30 scrsz(3)/2 scrsz(4)/2]);

    %Correr anlisis
    y = abs(fft(handles.valores));
    bar (y(2:floor(length(y)/2)),'k');  
    xlabel('Frecuencia'); ylabel('Potencia'); title(strcat('Anlisis espectral de: "','',handles.nomb,'"'));
    set(gca,'XTick',0:10:length(y)/2); set(gca,'FontSize',9); set(gca,'XTickLabelRotation',45);

% ----------------------- TFTC ----------------------------- %
function pushbuttonSTFT_Callback(hObject, eventdata, handles)
    guiTFTC();

%--------------------- Periodograma ------------------------%
function pushbuttonPrdg_Callback(hObject, eventdata, handles)
    guiPeriodograma();

%----------------------- Cosinor  -----------------------
function pushbuttonCosinor_Callback(hObject, eventdata, handles)
    guiCosinor();

%---------------------- Ondeletas  -----------------------
function pshbtnOndeletas_Callback(hObject, eventdata, handles)
    guiOndeletas();
    
    function actu (src,event,handles)
        guidata(BiCA,handles);
